@@ -14,13 +14,15 @@ typedef struct X11PropertyData
 {
     FF_LIBRARY_SYMBOL(XInternAtom)
     FF_LIBRARY_SYMBOL(XGetWindowProperty)
+    FF_LIBRARY_SYMBOL(XServerVendor)
     FF_LIBRARY_SYMBOL(XFree)
 } X11PropertyData;
 
-static bool x11InitPropertyData(void* libraryHandle, X11PropertyData* propertyData)
+static bool x11InitPropertyData(FF_MAYBE_UNUSED void* libraryHandle, X11PropertyData* propertyData)
 {
     FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XInternAtom, false)
     FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XGetWindowProperty, false)
+    FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XServerVendor, false)
     FF_LIBRARY_LOAD_SYMBOL_PTR(libraryHandle, propertyData, XFree, false)
 
     return true;
@@ -62,6 +64,16 @@ static void x11DetectWMFromEWMH(X11PropertyData* data, Display* display, FFDispl
     data->ffXFree(wmWindow);
 }
 
+static void x11FetchServerVendor(X11PropertyData* data, Display* display, FFDisplayServerResult* result)
+{
+    const char* serverVendor = data->ffXServerVendor(display);
+    if (serverVendor && !ffStrEquals(serverVendor, "The X.Org Foundation")) {
+        ffStrbufSetS(&result->wmProtocolName, serverVendor);
+    }
+
+
+}
+
 typedef struct XrandrData
 {
     FF_LIBRARY_SYMBOL(XInternAtom)
@@ -147,11 +159,10 @@ static bool xrandrHandleCrtc(XrandrData* data, XRROutputInfo* output, FFstrbuf*
         "xlib-randr-crtc"
     );
 
-    if (edidLength)
+    if (item && edidLength)
     {
         item->hdrStatus = ffEdidGetHdrCompatible(edidData, edidLength) ? FF_DISPLAY_HDR_STATUS_SUPPORTED : FF_DISPLAY_HDR_STATUS_UNSUPPORTED;
         ffEdidGetSerialAndManufactureDate(edidData, &item->serial, &item->manufactureYear, &item->manufactureWeek);
-        ffEdidGetPhysicalSize(edidData, &item->physicalWidth, &item->physicalHeight);
     }
 
     data->ffXRRFreeCrtcInfo(crtcInfo);
@@ -232,7 +243,7 @@ static bool xrandrHandleMonitors(XrandrData* data, Screen* screen)
     if(monitorInfos == NULL)
         return false;
 
-    bool foundAMonitor;
+    bool foundAMonitor = false;
 
     for(int i = 0; i < numberOfMonitors; i++)
     {
@@ -273,7 +284,7 @@ static void xrandrHandleScreen(XrandrData* data, Screen* screen)
         0,
         (uint32_t) WidthMMOfScreen(screen),
         (uint32_t) HeightMMOfScreen(screen),
-        "xlib_randr_screen"
+        "xlib-randr-screen"
     );
 }
 
@@ -306,8 +317,10 @@ const char* ffdsConnectXrandr(FFDisplayServerResult* result)
     if(data.display == NULL)
         return "XOpenDisplay() failed";
 
-    if(propertyDataInitialized && ScreenCount(data.display) > 0)
+    if(propertyDataInitialized && ScreenCount(data.display) > 0) {
         x11DetectWMFromEWMH(&propertyData, data.display, result);
+        x11FetchServerVendor(&propertyData, data.display, result);
+    }
 
     data.result = result;
 
