@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_HOST_MODULE_NAME "Host"
 
 void ffPrintHost(FFHostOptions* options);
 void ffInitHostOptions(FFHostOptions* options);
 void ffDestroyHostOptions(FFHostOptions* options);
+
+extern FFModuleBaseInfo ffHostModuleInfo;
