@@ -1,9 +1,11 @@
 #pragma once
 
-#include "fastfetch.h"
+#include "option.h"
 
 #define FF_LM_MODULE_NAME "LM"
 
 void ffPrintLM(FFLMOptions* options);
 void ffInitLMOptions(FFLMOptions* options);
 void ffDestroyLMOptions(FFLMOptions* options);
+
+extern FFModuleBaseInfo ffLMModuleInfo;
