@@ -17,7 +17,7 @@ void ffOptionsInitLogo(FFOptionsLogo* options)
     options->printRemaining = true;
     options->preserveAspectRatio = false;
     options->recache = false;
-    options->separate = false;
+    options->position = FF_LOGO_POSITION_LEFT;
 
     options->chafaFgOnly = false;
     ffStrbufInitStatic(&options->chafaSymbols, "block+border+space-wide-inverted"); // Chafa default
@@ -60,9 +60,11 @@ logoType:
                 { "file-raw", FF_LOGO_TYPE_FILE_RAW },
                 { "data", FF_LOGO_TYPE_DATA },
                 { "data-raw", FF_LOGO_TYPE_DATA_RAW },
+                { "command-raw", FF_LOGO_TYPE_COMMAND_RAW },
                 { "sixel", FF_LOGO_TYPE_IMAGE_SIXEL },
                 { "kitty", FF_LOGO_TYPE_IMAGE_KITTY },
                 { "kitty-direct", FF_LOGO_TYPE_IMAGE_KITTY_DIRECT },
+                { "kitty-icat", FF_LOGO_TYPE_IMAGE_KITTY_ICAT },
                 { "iterm", FF_LOGO_TYPE_IMAGE_ITERM },
                 { "chafa", FF_LOGO_TYPE_IMAGE_CHAFA },
                 { "raw", FF_LOGO_TYPE_IMAGE_RAW },
@@ -111,7 +113,19 @@ logoType:
         else if(ffStrEqualsIgnCase(subKey, "recache"))
             options->recache = ffOptionParseBoolean(value);
         else if(ffStrEqualsIgnCase(subKey, "separate"))
-            options->separate = ffOptionParseBoolean(value);
+        {
+            fputs("--logo-separate has been renamed to --logo-position\n", stderr);
+            exit(477);
+        }
+        else if(ffStrEqualsIgnCase(subKey, "position"))
+        {
+            options->position = (FFLogoPosition) ffOptionParseEnum(key, value, (FFKeyValuePair[]) {
+                { "left", FF_LOGO_POSITION_LEFT },
+                { "right", FF_LOGO_POSITION_RIGHT },
+                { "top", FF_LOGO_POSITION_TOP },
+                {},
+            });
+        }
         else
             return false;
     }
@@ -160,6 +174,11 @@ logoType:
         ffOptionParseString(key, value, &options->source);
         options->type = FF_LOGO_TYPE_IMAGE_KITTY_DIRECT;
     }
+    else if(ffStrEqualsIgnCase(key, "--kitty-icat"))
+    {
+        ffOptionParseString(key, value, &options->source);
+        options->type = FF_LOGO_TYPE_IMAGE_KITTY_ICAT;
+    }
     else if(ffStrEqualsIgnCase(key, "--iterm"))
     {
         ffOptionParseString(key, value, &options->source);
@@ -244,20 +263,17 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
 
     if (yyjson_is_str(object))
     {
-        const char* value = yyjson_get_str(object);
-        ffStrbufSetS(&options->source, value);
+        ffStrbufSetJsonVal(&options->source, object);
         return NULL;
     }
 
     if (!yyjson_is_obj(object)) return "Property 'logo' must be an object";
 
-    yyjson_val *key_, *val;
+    yyjson_val *key, *val;
     size_t idx, max;
-    yyjson_obj_foreach(object, idx, max, key_, val)
+    yyjson_obj_foreach(object, idx, max, key, val)
     {
-        const char* key = yyjson_get_str(key_);
-
-        if (ffStrEqualsIgnCase(key, "type"))
+        if (unsafe_yyjson_equals_str(key, "type"))
         {
             int value;
             const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
@@ -268,9 +284,11 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
                 { "file-raw", FF_LOGO_TYPE_FILE_RAW },
                 { "data", FF_LOGO_TYPE_DATA },
                 { "data-raw", FF_LOGO_TYPE_DATA_RAW },
+                { "command-raw", FF_LOGO_TYPE_COMMAND_RAW },
                 { "sixel", FF_LOGO_TYPE_IMAGE_SIXEL },
                 { "kitty", FF_LOGO_TYPE_IMAGE_KITTY },
                 { "kitty-direct", FF_LOGO_TYPE_IMAGE_KITTY_DIRECT },
+                { "kitty-icat", FF_LOGO_TYPE_IMAGE_KITTY_ICAT },
                 { "iterm", FF_LOGO_TYPE_IMAGE_ITERM },
                 { "chafa", FF_LOGO_TYPE_IMAGE_CHAFA },
                 { "raw", FF_LOGO_TYPE_IMAGE_RAW },
@@ -282,22 +300,21 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
             options->type = (FFLogoType) value;
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "source"))
+        else if (unsafe_yyjson_equals_str(key, "source"))
         {
-            ffStrbufSetS(&options->source, yyjson_get_str(val));
+            ffStrbufSetJsonVal(&options->source, val);
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "color"))
+        else if (unsafe_yyjson_equals_str(key, "color"))
         {
             if (!yyjson_is_obj(val))
                 return "Property 'color' must be an object";
 
-            yyjson_val *key_c, *valc;
+            yyjson_val *keyc, *valc;
             size_t idxc, maxc;
-            yyjson_obj_foreach(val, idxc, maxc, key_c, valc)
+            yyjson_obj_foreach(val, idxc, maxc, keyc, valc)
             {
-                const char* keyc = yyjson_get_str(key_c);
-                uint32_t index = (uint32_t) strtoul(keyc, NULL, 10);
+                uint32_t index = (uint32_t) strtoul(unsafe_yyjson_get_str(keyc), NULL, 10);
                 if (index < 1 || index > FASTFETCH_LOGO_MAX_COLORS)
                     return "Keys of property 'color' must be a number between 1 to 9";
 
@@ -305,23 +322,33 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
             }
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "width"))
+        else if (unsafe_yyjson_equals_str(key, "width"))
         {
-            uint32_t value = (uint32_t) yyjson_get_uint(val);
-            if (value == 0)
-                return "Logo width must be a positive integer";
-            options->width = value;
+            if (yyjson_is_null(val))
+                options->width = 0;
+            else
+            {
+                uint32_t value = (uint32_t) yyjson_get_uint(val);
+                if (value == 0)
+                    return "Logo width must be a positive integer";
+                options->width = value;
+            }
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "height"))
+        else if (unsafe_yyjson_equals_str(key, "height"))
         {
-            uint32_t value = (uint32_t) yyjson_get_uint(val);
-            if (value == 0)
-                return "Logo height must be a positive integer";
-            options->height = value;
+            if (yyjson_is_null(val))
+                options->height = 0;
+            else
+            {
+                uint32_t value = (uint32_t) yyjson_get_uint(val);
+                if (value == 0)
+                    return "Logo height must be a positive integer";
+                options->height = value;
+            }
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "padding"))
+        else if (unsafe_yyjson_equals_str(key, "padding"))
         {
             if (!yyjson_is_obj(val))
                 return "Logo padding must be an object";
@@ -340,27 +367,36 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
             #undef FF_PARSE_PADDING_POSITON
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "printRemaining"))
+        else if (unsafe_yyjson_equals_str(key, "printRemaining"))
         {
             options->printRemaining = yyjson_get_bool(val);
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "preserveAspectRatio"))
+        else if (unsafe_yyjson_equals_str(key, "preserveAspectRatio"))
         {
             options->preserveAspectRatio = yyjson_get_bool(val);
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "recache"))
+        else if (unsafe_yyjson_equals_str(key, "recache"))
         {
             options->recache = yyjson_get_bool(val);
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "separate"))
+        else if (unsafe_yyjson_equals_str(key, "position"))
         {
-            options->separate = yyjson_get_bool(val);
+            int value;
+            const char* error = ffJsonConfigParseEnum(val, &value, (FFKeyValuePair[]) {
+                { "left", FF_LOGO_POSITION_LEFT },
+                { "top", FF_LOGO_POSITION_TOP },
+                { "right", FF_LOGO_POSITION_RIGHT },
+                {},
+            });
+
+            if (error) return error;
+            options->position = (FFLogoPosition) value;
             continue;
         }
-        else if (ffStrEqualsIgnCase(key, "chafa"))
+        else if (unsafe_yyjson_equals_str(key, "chafa"))
         {
             if (!yyjson_is_obj(val))
                 return "Chafa config must be an object";
@@ -371,7 +407,7 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
 
             yyjson_val* symbols = yyjson_obj_get(val, "symbols");
             if (symbols)
-                ffStrbufAppendS(&options->chafaSymbols, yyjson_get_str(symbols));
+                ffStrbufSetJsonVal(&options->chafaSymbols, symbols);
 
             yyjson_val* canvasMode = yyjson_obj_get(val, "canvasMode");
             if (canvasMode)
@@ -432,112 +468,106 @@ const char* ffOptionsParseLogoJsonConfig(FFOptionsLogo* options, yyjson_val* roo
 
 void ffOptionsGenerateLogoJsonConfig(FFOptionsLogo* options, yyjson_mut_doc* doc)
 {
-    __attribute__((__cleanup__(ffOptionsDestroyLogo))) FFOptionsLogo defaultOptions;
-    ffOptionsInitLogo(&defaultOptions);
-
     yyjson_mut_val* obj = yyjson_mut_obj(doc);
 
-    if (options->type != defaultOptions.type)
+    switch (options->type)
     {
-        switch (options->type)
-        {
-            case FF_LOGO_TYPE_NONE:
-                yyjson_mut_obj_add_null(doc, doc->root, "logo");
-                return;
-            case FF_LOGO_TYPE_BUILTIN:
-                yyjson_mut_obj_add_str(doc, obj, "type", "builtin");
-                break;
-            case FF_LOGO_TYPE_SMALL:
-                yyjson_mut_obj_add_str(doc, obj, "type", "small");
-                break;
-            case FF_LOGO_TYPE_FILE:
-                yyjson_mut_obj_add_str(doc, obj, "type", "file");
-                break;
-            case FF_LOGO_TYPE_FILE_RAW:
-                yyjson_mut_obj_add_str(doc, obj, "type", "file-raw");
-                break;
-            case FF_LOGO_TYPE_DATA:
-                yyjson_mut_obj_add_str(doc, obj, "type", "data");
-                break;
-            case FF_LOGO_TYPE_DATA_RAW:
-                yyjson_mut_obj_add_str(doc, obj, "type", "data-raw");
-                break;
-            case FF_LOGO_TYPE_IMAGE_SIXEL:
-                yyjson_mut_obj_add_str(doc, obj, "type", "sixel");
-                break;
-            case FF_LOGO_TYPE_IMAGE_KITTY:
-                yyjson_mut_obj_add_str(doc, obj, "type", "kitty");
-                break;
-            case FF_LOGO_TYPE_IMAGE_ITERM:
-                yyjson_mut_obj_add_str(doc, obj, "type", "iterm");
-                break;
-            case FF_LOGO_TYPE_IMAGE_CHAFA:
-                yyjson_mut_obj_add_str(doc, obj, "type", "chafa");
-                break;
-            case FF_LOGO_TYPE_IMAGE_RAW:
-                yyjson_mut_obj_add_str(doc, obj, "type", "raw");
-                break;
-            default:
-                yyjson_mut_obj_add_str(doc, obj, "type", "auto");
-                break;
-        }
+        case FF_LOGO_TYPE_NONE:
+            yyjson_mut_obj_add_null(doc, doc->root, "logo");
+            return;
+        case FF_LOGO_TYPE_BUILTIN:
+            yyjson_mut_obj_add_str(doc, obj, "type", "builtin");
+            break;
+        case FF_LOGO_TYPE_SMALL:
+            yyjson_mut_obj_add_str(doc, obj, "type", "small");
+            break;
+        case FF_LOGO_TYPE_FILE:
+            yyjson_mut_obj_add_str(doc, obj, "type", "file");
+            break;
+        case FF_LOGO_TYPE_FILE_RAW:
+            yyjson_mut_obj_add_str(doc, obj, "type", "file-raw");
+            break;
+        case FF_LOGO_TYPE_DATA:
+            yyjson_mut_obj_add_str(doc, obj, "type", "data");
+            break;
+        case FF_LOGO_TYPE_DATA_RAW:
+            yyjson_mut_obj_add_str(doc, obj, "type", "data-raw");
+            break;
+        case FF_LOGO_TYPE_COMMAND_RAW:
+            yyjson_mut_obj_add_str(doc, obj, "type", "command-raw");
+            break;
+        case FF_LOGO_TYPE_IMAGE_SIXEL:
+            yyjson_mut_obj_add_str(doc, obj, "type", "sixel");
+            break;
+        case FF_LOGO_TYPE_IMAGE_KITTY:
+            yyjson_mut_obj_add_str(doc, obj, "type", "kitty");
+            break;
+        case FF_LOGO_TYPE_IMAGE_KITTY_DIRECT:
+            yyjson_mut_obj_add_str(doc, obj, "type", "kitty-direct");
+            break;
+        case FF_LOGO_TYPE_IMAGE_KITTY_ICAT:
+            yyjson_mut_obj_add_str(doc, obj, "type", "kitty-icat");
+            break;
+        case FF_LOGO_TYPE_IMAGE_ITERM:
+            yyjson_mut_obj_add_str(doc, obj, "type", "iterm");
+            break;
+        case FF_LOGO_TYPE_IMAGE_CHAFA:
+            yyjson_mut_obj_add_str(doc, obj, "type", "chafa");
+            break;
+        case FF_LOGO_TYPE_IMAGE_RAW:
+            yyjson_mut_obj_add_str(doc, obj, "type", "raw");
+            break;
+        default:
+            yyjson_mut_obj_add_str(doc, obj, "type", "auto");
+            break;
     }
 
-    if (!ffStrbufEqual(&options->source, &defaultOptions.source))
-        yyjson_mut_obj_add_str(doc, obj, "source", options->source.chars);
+    yyjson_mut_obj_add_str(doc, obj, "source", options->source.chars);
 
     {
         yyjson_mut_val* color = yyjson_mut_obj(doc);
         for (int i = 0; i < FASTFETCH_LOGO_MAX_COLORS; i++)
         {
-            if (!ffStrbufEqual(&options->colors[i], &defaultOptions.colors[i]))
-            {
-                char c = (char)('1' + i);
-                yyjson_mut_obj_add(color, yyjson_mut_strncpy(doc, &c, 1), yyjson_mut_strbuf(doc, &options->colors[i]));
-            }
+            char c = (char)('1' + i);
+            yyjson_mut_obj_add(color, yyjson_mut_strncpy(doc, &c, 1), yyjson_mut_strbuf(doc, &options->colors[i]));
         }
-        if (yyjson_mut_obj_size(color) > 0)
-            yyjson_mut_obj_add_val(doc, obj, "color", color);
+        yyjson_mut_obj_add_val(doc, obj, "color", color);
     }
 
-    if (options->width != defaultOptions.width)
+    if (options->width == 0)
+        yyjson_mut_obj_add_null(doc, obj, "width");
+    else
         yyjson_mut_obj_add_uint(doc, obj, "width", options->width);
 
-    if (options->height != defaultOptions.height)
+    if (options->height == 0)
+        yyjson_mut_obj_add_null(doc, obj, "height");
+    else
         yyjson_mut_obj_add_uint(doc, obj, "height", options->height);
 
     {
-        yyjson_mut_val* padding = yyjson_mut_obj(doc);
-        if (options->paddingTop != defaultOptions.paddingTop)
-            yyjson_mut_obj_add_uint(doc, padding, "top", options->paddingTop);
-        if (options->paddingLeft != defaultOptions.paddingLeft)
-            yyjson_mut_obj_add_uint(doc, padding, "left", options->paddingLeft);
-        if (options->paddingRight != defaultOptions.paddingRight)
-            yyjson_mut_obj_add_uint(doc, padding, "right", options->paddingRight);
-
-        if (yyjson_mut_obj_size(padding) > 0)
-            yyjson_mut_obj_add_val(doc, obj, "padding", padding);
+        yyjson_mut_val* padding = yyjson_mut_obj_add_obj(doc, obj, "padding");
+        yyjson_mut_obj_add_uint(doc, padding, "top", options->paddingTop);
+        yyjson_mut_obj_add_uint(doc, padding, "left", options->paddingLeft);
+        yyjson_mut_obj_add_uint(doc, padding, "right", options->paddingRight);
     }
 
-    if (options->printRemaining != defaultOptions.printRemaining)
-        yyjson_mut_obj_add_bool(doc, obj, "printRemaining", options->printRemaining);
+    yyjson_mut_obj_add_bool(doc, obj, "printRemaining", options->printRemaining);
 
-    if (options->preserveAspectRatio != defaultOptions.preserveAspectRatio)
-        yyjson_mut_obj_add_bool(doc, obj, "preserveAspectRatio", options->preserveAspectRatio);
+    yyjson_mut_obj_add_bool(doc, obj, "preserveAspectRatio", options->preserveAspectRatio);
 
-    if (options->recache != defaultOptions.recache)
-        yyjson_mut_obj_add_bool(doc, obj, "recache", options->recache);
+    yyjson_mut_obj_add_bool(doc, obj, "recache", options->recache);
 
-    if (options->separate != defaultOptions.separate)
-        yyjson_mut_obj_add_bool(doc, obj, "separate", options->separate);
+    yyjson_mut_obj_add_str(doc, obj, "position", ((const char* []) {
+        "left",
+        "top",
+        "right",
+    })[options->position]);
 
     {
         yyjson_mut_val* chafa = yyjson_mut_obj(doc);
-        if (options->chafaFgOnly != defaultOptions.chafaFgOnly)
-            yyjson_mut_obj_add_bool(doc, chafa, "fgOnly", options->chafaFgOnly);
-        if (!ffStrbufEqual(&options->chafaSymbols, &defaultOptions.chafaSymbols))
-            yyjson_mut_obj_add_strbuf(doc, chafa, "symbols", &options->chafaSymbols);
-        if (options->chafaCanvasMode != defaultOptions.chafaCanvasMode && options->chafaCanvasMode <= 7)
+        yyjson_mut_obj_add_bool(doc, chafa, "fgOnly", options->chafaFgOnly);
+        yyjson_mut_obj_add_strbuf(doc, chafa, "symbols", &options->chafaSymbols);
+        if (options->chafaCanvasMode <= 7)
         {
             yyjson_mut_obj_add_str(doc, chafa, "canvasMode", ((const char* []) {
                 "TRUECOLOR",
@@ -550,14 +580,14 @@ void ffOptionsGenerateLogoJsonConfig(FFOptionsLogo* options, yyjson_mut_doc* doc
                 "INDEXED_16_8",
             })[options->chafaCanvasMode]);
         }
-        if (options->chafaColorSpace != defaultOptions.chafaColorSpace && options->chafaColorSpace <= 1)
+        if (options->chafaColorSpace <= 1)
         {
             yyjson_mut_obj_add_str(doc, chafa, "colorSpace", ((const char* []) {
                 "RGB",
                 "DIN99D",
             })[options->chafaColorSpace]);
         }
-        if (options->chafaDitherMode != defaultOptions.chafaDitherMode && options->chafaDitherMode <= 2)
+        if (options->chafaDitherMode <= 2)
         {
             yyjson_mut_obj_add_str(doc, chafa, "ditherMode", ((const char* []) {
                 "NONE",
@@ -566,10 +596,8 @@ void ffOptionsGenerateLogoJsonConfig(FFOptionsLogo* options, yyjson_mut_doc* doc
             })[options->chafaDitherMode]);
         }
 
-        if (yyjson_mut_obj_size(chafa) > 0)
-            yyjson_mut_obj_add_val(doc, obj, "chafa", chafa);
+        yyjson_mut_obj_add_val(doc, obj, "chafa", chafa);
     }
 
-    if (yyjson_mut_obj_size(obj) > 0)
-        yyjson_mut_obj_add_val(doc, doc->root, "logo", obj);
+    yyjson_mut_obj_add_val(doc, doc->root, "logo", obj);
 }
